@@ -1,10 +1,13 @@
 """Checker Manager and Checker classes."""
+import argparse
 import collections
 import errno
 import itertools
 import logging
+import multiprocessing.pool
 import signal
 import tokenize
+from typing import Any
 from typing import Dict
 from typing import List
 from typing import Optional
@@ -14,11 +17,10 @@ from flake8 import defaults
 from flake8 import exceptions
 from flake8 import processor
 from flake8 import utils
-
-try:
-    import multiprocessing.pool
-except ImportError:
-    multiprocessing = None  # type: ignore
+from flake8.discover_files import expand_paths
+from flake8.plugins.finder import Checkers
+from flake8.plugins.finder import LoadedPlugin
+from flake8.style_guide import StyleGuideManager
 
 Results = List[Tuple[str, int, int, str, Optional[str]]]
 
@@ -27,7 +29,7 @@ LOG = logging.getLogger(__name__)
 SERIAL_RETRY_ERRNOS = {
     # ENOSPC: Added by sigmavirus24
     # > On some operating systems (OSX), multiprocessing may cause an
-    # > ENOSPC error while trying to trying to create a Semaphore.
+    # > ENOSPC error while trying to create a Semaphore.
     # > In those cases, we should replace the customized Queue Report
     # > class with pep8's StandardReport class to ensure users don't run
     # > into this problem.
@@ -40,11 +42,6 @@ SERIAL_RETRY_ERRNOS = {
 }
 
 
-def _multiprocessing_is_fork():  # type () -> bool
-    """Class state is only preserved when using the `fork` strategy."""
-    return multiprocessing and multiprocessing.get_start_method() == "fork"
-
-
 class Manager:
     """Manage the parallelism and checker instances for each plugin and file.
 
@@ -64,24 +61,15 @@ class Manager:
       together and make our output deterministic.
     """
 
-    def __init__(self, style_guide, arguments, checker_plugins):
-        """Initialize our Manager instance.
-
-        :param style_guide:
-            The instantiated style guide for this instance of Flake8.
-        :type style_guide:
-            flake8.style_guide.StyleGuide
-        :param list arguments:
-            The extra arguments parsed from the CLI (if any)
-        :param checker_plugins:
-            The plugins representing checks parsed from entry-points.
-        :type checker_plugins:
-            flake8.plugins.manager.Checkers
-        """
-        self.arguments = arguments
+    def __init__(
+        self,
+        style_guide: StyleGuideManager,
+        plugins: Checkers,
+    ) -> None:
+        """Initialize our Manager instance."""
         self.style_guide = style_guide
         self.options = style_guide.options
-        self.checks = checker_plugins
+        self.plugins = plugins
         self.jobs = self._job_count()
         self._all_checkers: List[FileChecker] = []
         self.checkers: List[FileChecker] = []
@@ -95,7 +83,7 @@ class Manager:
             itertools.chain(self.options.exclude, self.options.extend_exclude)
         )
 
-    def _process_statistics(self):
+    def _process_statistics(self) -> None:
         for checker in self.checkers:
             for statistic in defaults.STATISTIC_NAMES:
                 self.statistics[statistic] += checker.statistics[statistic]
@@ -111,14 +99,16 @@ class Manager:
         # - we're processing a diff, which again does not work well with
         #   multiprocessing and which really shouldn't require multiprocessing
         # - the user provided some awful input
-        if not _multiprocessing_is_fork():
+
+        # class state is only preserved when using the `fork` strategy.
+        if multiprocessing.get_start_method() != "fork":
             LOG.warning(
                 "The multiprocessing module is not available. "
                 "Ignoring --jobs arguments."
             )
             return 0
 
-        if utils.is_using_stdin(self.arguments):
+        if utils.is_using_stdin(self.options.filenames):
             LOG.warning(
                 "The --jobs option is not compatible with supplying "
                 "input using - . Ignoring --jobs arguments."
@@ -148,7 +138,7 @@ class Manager:
         # it to an integer
         return jobs.n_jobs
 
-    def _handle_results(self, filename, results):
+    def _handle_results(self, filename: str, results: Results) -> int:
         style_guide = self.style_guide
         reported_results_count = 0
         for (error_code, line_number, column, text, physical_line) in results:
@@ -162,70 +152,24 @@ class Manager:
             )
         return reported_results_count
 
-    def is_path_excluded(self, path: str) -> bool:
-        """Check if a path is excluded.
-
-        :param str path:
-            Path to check against the exclude patterns.
-        :returns:
-            True if there are exclude patterns and the path matches,
-            otherwise False.
-        :rtype:
-            bool
-        """
-        if path == "-":
-            if self.options.stdin_display_name == "stdin":
-                return False
-            path = self.options.stdin_display_name
-
-        return utils.matches_filename(
-            path,
-            patterns=self.exclude,
-            log_message='"%(path)s" has %(whether)sbeen excluded',
-            logger=LOG,
-        )
-
     def make_checkers(self, paths: Optional[List[str]] = None) -> None:
         """Create checkers for each file."""
         if paths is None:
-            paths = self.arguments
+            paths = self.options.filenames
 
-        if not paths:
-            paths = ["."]
-
-        filename_patterns = self.options.filename
-        running_from_diff = self.options.diff
-
-        # NOTE(sigmavirus24): Yes this is a little unsightly, but it's our
-        # best solution right now.
-        def should_create_file_checker(filename, argument):
-            """Determine if we should create a file checker."""
-            matches_filename_patterns = utils.fnmatch(
-                filename, filename_patterns
-            )
-            is_stdin = filename == "-"
-            # NOTE(sigmavirus24): If a user explicitly specifies something,
-            # e.g, ``flake8 bin/script`` then we should run Flake8 against
-            # that. Since should_create_file_checker looks to see if the
-            # filename patterns match the filename, we want to skip that in
-            # the event that the argument and the filename are identical.
-            # If it was specified explicitly, the user intended for it to be
-            # checked.
-            explicitly_provided = not running_from_diff and (
-                argument == filename
-            )
-            return (
-                explicitly_provided or matches_filename_patterns
-            ) or is_stdin
-
-        checks = self.checks.to_dictionary()
         self._all_checkers = [
-            FileChecker(filename, checks, self.options)
-            for argument in paths
-            for filename in utils.filenames_from(
-                argument, self.is_path_excluded
+            FileChecker(
+                filename=filename,
+                plugins=self.plugins,
+                options=self.options,
+            )
+            for filename in expand_paths(
+                paths=paths,
+                stdin_display_name=self.options.stdin_display_name,
+                filename_patterns=self.options.filename,
+                exclude=self.exclude,
+                is_running_from_diff=self.options.diff,
             )
-            if should_create_file_checker(filename, argument)
         ]
         self.checkers = [c for c in self._all_checkers if c.should_process]
         LOG.info("Checking %d files", len(self.checkers))
@@ -238,8 +182,6 @@ class Manager:
 
         :returns:
             A tuple of the total results found and the results reported.
-        :rtype:
-            tuple(int, int)
         """
         results_reported = results_found = 0
         for checker in self._all_checkers:
@@ -301,8 +243,7 @@ class Manager:
         or whether to run them in serial.
 
         If running the checks in parallel causes a problem (e.g.,
-        https://github.com/pycqa/flake8/issues/117) this also implements
-        fallback to serial processing.
+        :issue:`117`) this also implements fallback to serial processing.
         """
         try:
             if self.jobs > 1 and len(self.checkers) > 1:
@@ -313,17 +254,17 @@ class Manager:
             LOG.warning("Flake8 was interrupted by the user")
             raise exceptions.EarlyQuit("Early quit while running checks")
 
-    def start(self, paths=None):
+    def start(self, paths: Optional[List[str]] = None) -> None:
         """Start checking files.
 
-        :param list paths:
+        :param paths:
             Path names to check. This is passed directly to
             :meth:`~Manager.make_checkers`.
         """
         LOG.info("Making checkers")
         self.make_checkers(paths)
 
-    def stop(self):
+    def stop(self) -> None:
         """Stop checking files."""
         self._process_statistics()
 
@@ -331,23 +272,17 @@ class Manager:
 class FileChecker:
     """Manage running checks for a file and aggregate the results."""
 
-    def __init__(self, filename, checks, options):
-        """Initialize our file checker.
-
-        :param str filename:
-            Name of the file to check.
-        :param checks:
-            The plugins registered to check the file.
-        :type checks:
-            dict
-        :param options:
-            Parsed option values from config and command-line.
-        :type options:
-            argparse.Namespace
-        """
+    def __init__(
+        self,
+        *,
+        filename: str,
+        plugins: Checkers,
+        options: argparse.Namespace,
+    ) -> None:
+        """Initialize our file checker."""
         self.options = options
         self.filename = filename
-        self.checks = checks
+        self.plugins = plugins
         self.results: Results = []
         self.statistics = {
             "tokens": 0,
@@ -400,29 +335,29 @@ class FileChecker:
         self.results.append((error_code, line_number, column, text, line))
         return error_code
 
-    def run_check(self, plugin, **arguments):
+    def run_check(self, plugin: LoadedPlugin, **arguments: Any) -> Any:
         """Run the check in a single plugin."""
-        LOG.debug("Running %r with %r", plugin, arguments)
         assert self.processor is not None
         try:
-            self.processor.keyword_arguments_for(
-                plugin["parameters"], arguments
+            params = self.processor.keyword_arguments_for(
+                plugin.parameters, arguments
             )
         except AttributeError as ae:
-            LOG.error("Plugin requested unknown parameters.")
             raise exceptions.PluginRequestedUnknownParameters(
-                plugin=plugin, exception=ae
+                plugin_name=plugin.display_name, exception=ae
             )
         try:
-            return plugin["plugin"](**arguments)
+            return plugin.obj(**arguments, **params)
         except Exception as all_exc:
             LOG.critical(
                 "Plugin %s raised an unexpected exception",
-                plugin["name"],
+                plugin.display_name,
                 exc_info=True,
             )
             raise exceptions.PluginExecutionFailed(
-                plugin=plugin, exception=all_exc
+                filename=self.filename,
+                plugin_name=plugin.display_name,
+                exception=all_exc,
             )
 
     @staticmethod
@@ -489,7 +424,7 @@ class FileChecker:
         assert self.processor is not None
         ast = self.processor.build_ast()
 
-        for plugin in self.checks["ast_plugins"]:
+        for plugin in self.plugins.tree:
             checker = self.run_check(plugin, tree=ast)
             # If the plugin uses a class, call the run method of it, otherwise
             # the call should return something iterable itself
@@ -505,7 +440,7 @@ class FileChecker:
                     text=text,
                 )
 
-    def run_logical_checks(self):
+    def run_logical_checks(self) -> None:
         """Run all checks expecting a logical line."""
         assert self.processor is not None
         comments, logical_line, mapping = self.processor.build_logical_line()
@@ -515,7 +450,7 @@ class FileChecker:
 
         LOG.debug('Logical line: "%s"', logical_line.rstrip())
 
-        for plugin in self.checks["logical_line_plugins"]:
+        for plugin in self.plugins.logical_line:
             self.processor.update_checker_state_for(plugin)
             results = self.run_check(plugin, logical_line=logical_line) or ()
             for offset, text in results:
@@ -531,13 +466,13 @@ class FileChecker:
 
         self.processor.next_logical_line()
 
-    def run_physical_checks(self, physical_line):
+    def run_physical_checks(self, physical_line: str) -> None:
         """Run all checks for a given physical line.
 
         A single physical check may return multiple errors.
         """
         assert self.processor is not None
-        for plugin in self.checks["physical_line_plugins"]:
+        for plugin in self.plugins.physical_line:
             self.processor.update_checker_state_for(plugin)
             result = self.run_check(plugin, physical_line=physical_line)
 
@@ -562,7 +497,7 @@ class FileChecker:
                         text=text,
                     )
 
-    def process_tokens(self):
+    def process_tokens(self) -> None:
         """Process tokens and trigger checks.
 
         Instead of using this directly, you should use
@@ -577,7 +512,6 @@ class FileChecker:
             statistics["tokens"] += 1
             self.check_physical_eol(token, prev_physical)
             token_type, text = token[0:2]
-            processor.log_token(LOG, token)
             if token_type == tokenize.OP:
                 parens = processor.count_parentheses(parens, text)
             elif parens == 0:
@@ -606,7 +540,7 @@ class FileChecker:
         self.statistics["logical lines"] = logical_lines
         return self.filename, self.results, self.statistics
 
-    def handle_newline(self, token_type):
+    def handle_newline(self, token_type: int) -> None:
         """Handle the logic when encountering a newline token."""
         assert self.processor is not None
         if token_type == tokenize.NEWLINE:
@@ -620,7 +554,7 @@ class FileChecker:
             self.run_logical_checks()
 
     def check_physical_eol(
-        self, token: processor._Token, prev_physical: str
+        self, token: tokenize.TokenInfo, prev_physical: str
     ) -> None:
         """Run physical checks if and only if it is at the end of the line."""
         assert self.processor is not None
@@ -648,7 +582,7 @@ class FileChecker:
             line_no = token[2][0]
             with self.processor.inside_multiline(line_number=line_no):
                 for line in self.processor.split_line(token):
-                    self.run_physical_checks(line + "\n")
+                    self.run_physical_checks(line)
 
 
 def _pool_init() -> None:
@@ -671,7 +605,7 @@ def _try_initialize_processpool(
     return None
 
 
-def calculate_pool_chunksize(num_checkers, num_jobs):
+def calculate_pool_chunksize(num_checkers: int, num_jobs: int) -> int:
     """Determine the chunksize for the multiprocessing Pool.
 
     - For chunksize, see: https://docs.python.org/3/library/multiprocessing.html#multiprocessing.pool.Pool.imap  # noqa
@@ -683,7 +617,7 @@ def calculate_pool_chunksize(num_checkers, num_jobs):
     return max(num_checkers // (num_jobs * 2), 1)
 
 
-def _run_checks(checker):
+def _run_checks(checker: FileChecker) -> Tuple[str, Results, Dict[str, int]]:
     return checker.run_checks()
 
 
