@@ -9,14 +9,14 @@ EXCLUDE = (
     ".git",
     "__pycache__",
     ".tox",
+    ".nox",
     ".eggs",
     "*.egg",
 )
 IGNORE = ("E121", "E123", "E126", "E226", "E24", "E704", "W503", "W504")
 SELECT = ("E", "F", "W", "C90")
 MAX_LINE_LENGTH = 79
-
-TRUTHY_VALUES = {"true", "1", "t"}
+INDENT_SIZE = 4
 
 # Other constants
 WHITESPACE = frozenset(" \t")
@@ -35,7 +35,7 @@ NOQA_INLINE_REGEXP = re.compile(
     # We do not want to capture the ``: `` that follows ``noqa``
     # We do not care about the casing of ``noqa``
     # We want a comma-separated list of errors
-    # https://regex101.com/r/4XUuax/2 full explenation of the regex
+    # https://regex101.com/r/4XUuax/2 full explanation of the regex
     r"# noqa(?::[\s]?(?P<codes>([A-Z]+[0-9]+(?:[,\s]+)?)+))?",
     re.IGNORECASE,
 )
