@@ -1,18 +1,18 @@
 """Command-line implementation of flake8."""
 import sys
-from typing import List
 from typing import Optional
+from typing import Sequence
 
 from flake8.main import application
 
 
-def main(argv: Optional[List[str]] = None) -> None:
+def main(argv: Optional[Sequence[str]] = None) -> int:
     """Execute the main bit of the application.
 
     This handles the creation of an instance of :class:`Application`, runs it,
     and then exits the application.
 
-    :param list argv:
+    :param argv:
         The arguments to be passed to the application for parsing.
     """
     if argv is None:
@@ -20,4 +20,4 @@ def main(argv: Optional[List[str]] = None) -> None:
 
     app = application.Application()
     app.run(argv)
-    app.exit()
+    return app.exit_code()
