@@ -1,19 +1,120 @@
 """Contains the logic for all of the default options for Flake8."""
+import argparse
+
 from flake8 import defaults
-from flake8.main import debug
-from flake8.main import vcs
+from flake8.options.manager import OptionManager
 
 
-def register_default_options(option_manager):
+def stage1_arg_parser() -> argparse.ArgumentParser:
+    """Register the preliminary options on our OptionManager.
+
+    The preliminary options include:
+
+    - ``-v``/``--verbose``
+    - ``--output-file``
+    - ``--append-config``
+    - ``--config``
+    - ``--isolated``
+    - ``--enable-extensions``
+    """
+    parser = argparse.ArgumentParser(add_help=False)
+
+    parser.add_argument(
+        "-v",
+        "--verbose",
+        default=0,
+        action="count",
+        help="Print more information about what is happening in flake8. "
+        "This option is repeatable and will increase verbosity each "
+        "time it is repeated.",
+    )
+
+    parser.add_argument(
+        "--output-file", default=None, help="Redirect report to a file."
+    )
+
+    # Config file options
+
+    parser.add_argument(
+        "--append-config",
+        action="append",
+        default=[],
+        help="Provide extra config files to parse in addition to the files "
+        "found by Flake8 by default. These files are the last ones read "
+        "and so they take the highest precedence when multiple files "
+        "provide the same option.",
+    )
+
+    parser.add_argument(
+        "--config",
+        default=None,
+        help="Path to the config file that will be the authoritative config "
+        "source. This will cause Flake8 to ignore all other "
+        "configuration files.",
+    )
+
+    parser.add_argument(
+        "--isolated",
+        default=False,
+        action="store_true",
+        help="Ignore all configuration files.",
+    )
+
+    # Plugin enablement options
+
+    parser.add_argument(
+        "--enable-extensions",
+        help="Enable plugins and extensions that are otherwise disabled "
+        "by default",
+    )
+
+    parser.add_argument(
+        "--require-plugins",
+        help="Require specific plugins to be installed before running",
+    )
+
+    return parser
+
+
+class JobsArgument:
+    """Type callback for the --jobs argument."""
+
+    def __init__(self, arg: str) -> None:
+        """Parse and validate the --jobs argument.
+
+        :param arg: The argument passed by argparse for validation
+        """
+        self.is_auto = False
+        self.n_jobs = -1
+        if arg == "auto":
+            self.is_auto = True
+        elif arg.isdigit():
+            self.n_jobs = int(arg)
+        else:
+            raise argparse.ArgumentTypeError(
+                f"{arg!r} must be 'auto' or an integer.",
+            )
+
+    def __repr__(self) -> str:
+        """Representation for debugging."""
+        return f"{type(self).__name__}({str(self)!r})"
+
+    def __str__(self) -> str:
+        """Format our JobsArgument class."""
+        return "auto" if self.is_auto else str(self.n_jobs)
+
+
+def register_default_options(option_manager: OptionManager) -> None:
     """Register the default options on our OptionManager.
 
     The default options include:
 
-    - ``-v``/``--verbose``
     - ``-q``/``--quiet``
+    - ``--color``
     - ``--count``
     - ``--diff``
     - ``--exclude``
+    - ``--extend-exclude``
     - ``--filename``
     - ``--format``
     - ``--hang-closing``
@@ -22,34 +123,20 @@ def register_default_options(option_manager):
     - ``--per-file-ignores``
     - ``--max-line-length``
     - ``--max-doc-length``
+    - ``--indent-size``
     - ``--select``
+    - ``--extend-select``
     - ``--disable-noqa``
     - ``--show-source``
     - ``--statistics``
-    - ``--enable-extensions``
     - ``--exit-zero``
     - ``-j``/``--jobs``
-    - ``--output-file``
     - ``--tee``
-    - ``--append-config``
-    - ``--config``
-    - ``--isolated``
     - ``--benchmark``
     - ``--bug-report``
     """
     add_option = option_manager.add_option
 
-    # pep8 options
-    add_option(
-        "-v",
-        "--verbose",
-        default=0,
-        action="count",
-        parse_from_config=True,
-        help="Print more information about what is happening in flake8."
-        " This option is repeatable and will increase verbosity each "
-        "time it is repeated.",
-    )
     add_option(
         "-q",
         "--quiet",
@@ -59,19 +146,26 @@ def register_default_options(option_manager):
         help="Report only file names, or nothing. This option is repeatable.",
     )
 
+    add_option(
+        "--color",
+        choices=("auto", "always", "never"),
+        default="auto",
+        help="Whether to use color in output.  Defaults to `%(default)s`.",
+    )
+
     add_option(
         "--count",
         action="store_true",
         parse_from_config=True,
-        help="Print total number of errors and warnings to standard error and"
-        " set the exit code to 1 if total is not empty.",
+        help="Print total number of errors to standard output and "
+        "set the exit code to 1 if total is not empty.",
     )
 
     add_option(
         "--diff",
         action="store_true",
-        help="Report changes only within line number ranges in the unified "
-        "diff provided on standard in by the user.",
+        help="(DEPRECATED) Report changes only within line number ranges in "
+        "the unified diff provided on standard in by the user.",
     )
 
     add_option(
@@ -81,8 +175,19 @@ def register_default_options(option_manager):
         comma_separated_list=True,
         parse_from_config=True,
         normalize_paths=True,
-        help="Comma-separated list of files or directories to exclude."
-        " (Default: %default)",
+        help="Comma-separated list of files or directories to exclude. "
+        "(Default: %(default)s)",
+    )
+
+    add_option(
+        "--extend-exclude",
+        metavar="patterns",
+        default="",
+        parse_from_config=True,
+        comma_separated_list=True,
+        normalize_paths=True,
+        help="Comma-separated list of files or directories to add to the list "
+        "of excluded ones.",
     )
 
     add_option(
@@ -92,15 +197,15 @@ def register_default_options(option_manager):
         parse_from_config=True,
         comma_separated_list=True,
         help="Only check for filenames matching the patterns in this comma-"
-        "separated list. (Default: %default)",
+        "separated list. (Default: %(default)s)",
     )
 
     add_option(
         "--stdin-display-name",
         default="stdin",
-        help="The name used when reporting errors from code passed via stdin."
-        " This is useful for editors piping the file contents to flake8."
-        " (Default: %default)",
+        help="The name used when reporting errors from code passed via stdin. "
+        "This is useful for editors piping the file contents to flake8. "
+        "(Default: %(default)s)",
     )
 
     # TODO(sigmavirus24): Figure out --first/--repeat
@@ -120,28 +225,29 @@ def register_default_options(option_manager):
         "--hang-closing",
         action="store_true",
         parse_from_config=True,
-        help="Hang closing bracket instead of matching indentation of opening"
-        " bracket's line.",
+        help="Hang closing bracket instead of matching indentation of opening "
+        "bracket's line.",
     )
 
     add_option(
         "--ignore",
         metavar="errors",
-        default=",".join(defaults.IGNORE),
         parse_from_config=True,
         comma_separated_list=True,
-        help="Comma-separated list of errors and warnings to ignore (or skip)."
-        " For example, ``--ignore=E4,E51,W234``. (Default: %default)",
+        help=(
+            f"Comma-separated list of error codes to ignore (or skip). "
+            f"For example, ``--ignore=E4,E51,W234``. "
+            f"(Default: {','.join(defaults.IGNORE)})"
+        ),
     )
 
     add_option(
         "--extend-ignore",
         metavar="errors",
-        default="",
         parse_from_config=True,
         comma_separated_list=True,
-        help="Comma-separated list of errors and warnings to add to the list"
-        " of ignored ones. For example, ``--extend-ignore=E4,E51,W234``.",
+        help="Comma-separated list of error codes to add to the list of "
+        "ignored ones. For example, ``--extend-ignore=E4,E51,W234``.",
     )
 
     add_option(
@@ -157,32 +263,53 @@ def register_default_options(option_manager):
 
     add_option(
         "--max-line-length",
-        type="int",
+        type=int,
         metavar="n",
         default=defaults.MAX_LINE_LENGTH,
         parse_from_config=True,
         help="Maximum allowed line length for the entirety of this run. "
-        "(Default: %default)",
+        "(Default: %(default)s)",
     )
 
     add_option(
         "--max-doc-length",
-        type="int",
+        type=int,
         metavar="n",
         default=None,
         parse_from_config=True,
         help="Maximum allowed doc line length for the entirety of this run. "
-        "(Default: %default)",
+        "(Default: %(default)s)",
+    )
+    add_option(
+        "--indent-size",
+        type=int,
+        metavar="n",
+        default=defaults.INDENT_SIZE,
+        parse_from_config=True,
+        help="Number of spaces used for indentation (Default: %(default)s)",
     )
 
     add_option(
         "--select",
         metavar="errors",
-        default=",".join(defaults.SELECT),
         parse_from_config=True,
         comma_separated_list=True,
-        help="Comma-separated list of errors and warnings to enable."
-        " For example, ``--select=E4,E51,W234``. (Default: %default)",
+        help=(
+            f"Comma-separated list of error codes to enable. "
+            f"For example, ``--select=E4,E51,W234``. "
+            f"(Default: {','.join(defaults.SELECT)})"
+        ),
+    )
+
+    add_option(
+        "--extend-select",
+        metavar="errors",
+        parse_from_config=True,
+        comma_separated_list=True,
+        help=(
+            "Comma-separated list of error codes to add to the list "
+            "of selected ones. For example, ``--extend-select=E4,E51,W234``."
+        ),
     )
 
     add_option(
@@ -202,24 +329,22 @@ def register_default_options(option_manager):
         parse_from_config=True,
         help="Show the source generate each error or warning.",
     )
+    add_option(
+        "--no-show-source",
+        action="store_false",
+        dest="show_source",
+        parse_from_config=False,
+        help="Negate --show-source",
+    )
 
     add_option(
         "--statistics",
         action="store_true",
         parse_from_config=True,
-        help="Count errors and warnings.",
+        help="Count errors.",
     )
 
     # Flake8 options
-    add_option(
-        "--enable-extensions",
-        default="",
-        parse_from_config=True,
-        comma_separated_list=True,
-        type="string",
-        help="Enable plugins and extensions that are otherwise disabled "
-        "by default",
-    )
 
     add_option(
         "--exit-zero",
@@ -227,35 +352,16 @@ def register_default_options(option_manager):
         help='Exit with status code "0" even if there are errors.',
     )
 
-    add_option(
-        "--install-hook",
-        action="callback",
-        type="choice",
-        choices=vcs.choices(),
-        callback=vcs.install,
-        help="Install a hook that is run prior to a commit for the supported "
-        "version control system.",
-    )
-
     add_option(
         "-j",
         "--jobs",
-        type="string",
         default="auto",
         parse_from_config=True,
+        type=JobsArgument,
         help="Number of subprocesses to use to run checks in parallel. "
         'This is ignored on Windows. The default, "auto", will '
-        "auto-detect the number of processors available to use."
-        " (Default: %default)",
-    )
-
-    add_option(
-        "--output-file",
-        default=None,
-        type="string",
-        parse_from_config=True,
-        # callback=callbacks.redirect_stdout,
-        help="Redirect report to a file.",
+        "auto-detect the number of processors available to use. "
+        "(Default: %(default)s)",
     )
 
     add_option(
@@ -266,32 +372,6 @@ def register_default_options(option_manager):
         help="Write to stdout and output-file.",
     )
 
-    # Config file options
-
-    add_option(
-        "--append-config",
-        action="append",
-        help="Provide extra config files to parse in addition to the files "
-        "found by Flake8 by default. These files are the last ones read "
-        "and so they take the highest precedence when multiple files "
-        "provide the same option.",
-    )
-
-    add_option(
-        "--config",
-        default=None,
-        help="Path to the config file that will be the authoritative config "
-        "source. This will cause Flake8 to ignore all other "
-        "configuration files.",
-    )
-
-    add_option(
-        "--isolated",
-        default=False,
-        action="store_true",
-        help="Ignore all configuration files.",
-    )
-
     # Benchmarking
 
     add_option(
@@ -305,8 +385,6 @@ def register_default_options(option_manager):
 
     add_option(
         "--bug-report",
-        action="callback",
-        callback=debug.print_information,
-        callback_kwargs={"option_manager": option_manager},
+        action="store_true",
         help="Print information necessary when preparing a bug report",
     )
