@@ -1,27 +1,32 @@
 """Module containing our file processor that tokenizes a file for checks."""
+import argparse
+import ast
 import contextlib
 import logging
-import sys
 import tokenize
+from typing import Any
+from typing import Dict
+from typing import Generator
+from typing import List
+from typing import Optional
+from typing import Tuple
 
-import flake8
 from flake8 import defaults
-from flake8 import exceptions
 from flake8 import utils
+from flake8.plugins.finder import LoadedPlugin
 
 LOG = logging.getLogger(__name__)
-PyCF_ONLY_AST = 1024
 NEWLINE = frozenset([tokenize.NL, tokenize.NEWLINE])
-# Work around Python < 2.6 behaviour, which does not generate NL after
-# a comment which is on a line by itself.
-COMMENT_WITH_NL = tokenize.generate_tokens(["#\n"].pop).send(None)[1] == "#\n"
 
 SKIP_TOKENS = frozenset(
     [tokenize.NL, tokenize.NEWLINE, tokenize.INDENT, tokenize.DEDENT]
 )
 
+_LogicalMapping = List[Tuple[int, Tuple[int, int]]]
+_Logical = Tuple[List[str], List[str], _LogicalMapping]
 
-class FileProcessor(object):
+
+class FileProcessor:
     """Processes a file and holdes state.
 
     This processes a file by generating tokens, logical and physical lines,
@@ -37,6 +42,7 @@ class FileProcessor(object):
     - :attr:`line_number`
     - :attr:`logical_line`
     - :attr:`max_line_length`
+    - :attr:`max_doc_length`
     - :attr:`multiline`
     - :attr:`noqa`
     - :attr:`previous_indent_level`
@@ -48,17 +54,22 @@ class FileProcessor(object):
     - :attr:`verbose`
     """
 
-    def __init__(self, filename, options, lines=None):
+    #: always ``False``, included for compatibility
+    noqa = False
+
+    def __init__(
+        self,
+        filename: str,
+        options: argparse.Namespace,
+        lines: Optional[List[str]] = None,
+    ) -> None:
         """Initialice our file processor.
 
-        :param str filename:
-            Name of the file to process
+        :param filename: Name of the file to process
         """
         self.options = options
         self.filename = filename
-        self.lines = lines
-        if lines is None:
-            self.lines = self.read_lines()
+        self.lines = lines if lines is not None else self.read_lines()
         self.strip_utf_bom()
 
         # Defaults for public attributes
@@ -67,25 +78,27 @@ class FileProcessor(object):
         #: Number of blank lines
         self.blank_lines = 0
         #: Checker states for each plugin?
-        self._checker_states = {}
+        self._checker_states: Dict[str, Dict[Any, Any]] = {}
         #: Current checker state
-        self.checker_state = None
+        self.checker_state: Dict[Any, Any] = {}
         #: User provided option for hang closing
         self.hang_closing = options.hang_closing
         #: Character used for indentation
-        self.indent_char = None
+        self.indent_char: Optional[str] = None
         #: Current level of indentation
         self.indent_level = 0
+        #: Number of spaces used for indentation
+        self.indent_size = options.indent_size
         #: Line number in the file
         self.line_number = 0
         #: Current logical line
         self.logical_line = ""
         #: Maximum line length as configured by the user
         self.max_line_length = options.max_line_length
+        #: Maximum docstring / comment line length as configured by the user
+        self.max_doc_length = options.max_doc_length
         #: Whether the current physical line is multiline
         self.multiline = False
-        #: Whether or not we're observing NoQA
-        self.noqa = False
         #: Previous level of indentation
         self.previous_indent_level = 0
         #: Previous logical line
@@ -93,55 +106,51 @@ class FileProcessor(object):
         #: Previous unindented (i.e. top-level) logical line
         self.previous_unindented_logical_line = ""
         #: Current set of tokens
-        self.tokens = []
+        self.tokens: List[tokenize.TokenInfo] = []
         #: Total number of lines in the file
         self.total_lines = len(self.lines)
         #: Verbosity level of Flake8
         self.verbose = options.verbose
         #: Statistics dictionary
         self.statistics = {"logical lines": 0}
-        self._file_tokens = None
+        self._file_tokens: Optional[List[tokenize.TokenInfo]] = None
+        # map from line number to the line we'll search for `noqa` in
+        self._noqa_line_mapping: Optional[Dict[int, str]] = None
 
     @property
-    def file_tokens(self):
-        """Return the complete set of tokens for a file.
-
-        Accessing this attribute *may* raise an InvalidSyntax exception.
-
-        :raises: flake8.exceptions.InvalidSyntax
-        """
+    def file_tokens(self) -> List[tokenize.TokenInfo]:
+        """Return the complete set of tokens for a file."""
         if self._file_tokens is None:
             line_iter = iter(self.lines)
-            try:
-                self._file_tokens = list(
-                    tokenize.generate_tokens(lambda: next(line_iter))
-                )
-            except tokenize.TokenError as exc:
-                raise exceptions.InvalidSyntax(exc.message, exception=exc)
+            self._file_tokens = list(
+                tokenize.generate_tokens(lambda: next(line_iter))
+            )
 
         return self._file_tokens
 
     @contextlib.contextmanager
-    def inside_multiline(self, line_number):
+    def inside_multiline(
+        self, line_number: int
+    ) -> Generator[None, None, None]:
         """Context-manager to toggle the multiline attribute."""
         self.line_number = line_number
         self.multiline = True
         yield
         self.multiline = False
 
-    def reset_blank_before(self):
+    def reset_blank_before(self) -> None:
         """Reset the blank_before attribute to zero."""
         self.blank_before = 0
 
-    def delete_first_token(self):
+    def delete_first_token(self) -> None:
         """Delete the first token in the list of tokens."""
         del self.tokens[0]
 
-    def visited_new_blank_line(self):
+    def visited_new_blank_line(self) -> None:
         """Note that we visited a new blank line."""
         self.blank_lines += 1
 
-    def update_state(self, mapping):
+    def update_state(self, mapping: _LogicalMapping) -> None:
         """Update the indent level based on the logical line mapping."""
         (start_row, start_col) = mapping[0][1]
         start_line = self.lines[start_row - 1]
@@ -149,14 +158,14 @@ class FileProcessor(object):
         if self.blank_before < self.blank_lines:
             self.blank_before = self.blank_lines
 
-    def update_checker_state_for(self, plugin):
+    def update_checker_state_for(self, plugin: LoadedPlugin) -> None:
         """Update the checker_state attribute for the plugin."""
-        if "checker_state" in plugin["parameters"]:
+        if "checker_state" in plugin.parameters:
             self.checker_state = self._checker_states.setdefault(
-                plugin["name"], {}
+                plugin.entry_name, {}
             )
 
-    def next_logical_line(self):
+    def next_logical_line(self) -> None:
         """Record the previous logical line.
 
         This also resets the tokens list and the blank_lines count.
@@ -168,14 +177,14 @@ class FileProcessor(object):
                 self.previous_unindented_logical_line = self.logical_line
         self.blank_lines = 0
         self.tokens = []
-        self.noqa = False
 
-    def build_logical_line_tokens(self):
+    def build_logical_line_tokens(self) -> _Logical:
         """Build the mapping, comments, and logical line lists."""
         logical = []
         comments = []
+        mapping: _LogicalMapping = []
         length = 0
-        previous_row = previous_column = mapping = None
+        previous_row = previous_column = None
         for token_type, text, start, end, line in self.tokens:
             if token_type in SKIP_TOKENS:
                 continue
@@ -195,7 +204,7 @@ class FileProcessor(object):
                     if previous_text == "," or (
                         previous_text not in "{[(" and text not in "}])"
                     ):
-                        text = " " + text
+                        text = f" {text}"
                 elif previous_column != start_column:
                     text = line[previous_column:start_column] + text
             logical.append(text)
@@ -204,41 +213,45 @@ class FileProcessor(object):
             (previous_row, previous_column) = end
         return comments, logical, mapping
 
-    def build_ast(self):
+    def build_ast(self) -> ast.AST:
         """Build an abstract syntax tree from the list of lines."""
-        return compile("".join(self.lines), "", "exec", PyCF_ONLY_AST)
+        return ast.parse("".join(self.lines))
 
-    def build_logical_line(self):
+    def build_logical_line(self) -> Tuple[str, str, _LogicalMapping]:
         """Build a logical line from the current tokens list."""
         comments, logical, mapping_list = self.build_logical_line_tokens()
         joined_comments = "".join(comments)
         self.logical_line = "".join(logical)
-        if defaults.NOQA_INLINE_REGEXP.search(joined_comments):
-            self.noqa = True
         self.statistics["logical lines"] += 1
         return joined_comments, self.logical_line, mapping_list
 
-    def split_line(self, token):
+    def split_line(
+        self, token: tokenize.TokenInfo
+    ) -> Generator[str, None, None]:
         """Split a physical line's line based on new-lines.
 
         This also auto-increments the line number for the caller.
         """
-        for line in token[1].split("\n")[:-1]:
-            yield line
+        # intentionally don't include the last line, that line will be
+        # terminated later by a future end-of-line
+        for line_no in range(token.start[0], token.end[0]):
+            yield self.lines[line_no - 1]
             self.line_number += 1
 
-    def keyword_arguments_for(self, parameters, arguments=None):
+    def keyword_arguments_for(
+        self,
+        parameters: Dict[str, bool],
+        arguments: Dict[str, Any],
+    ) -> Dict[str, Any]:
         """Generate the keyword arguments for a list of parameters."""
-        if arguments is None:
-            arguments = {}
+        ret = {}
         for param, required in parameters.items():
             if param in arguments:
                 continue
             try:
-                arguments[param] = getattr(self, param)
-            except AttributeError as exc:
+                ret[param] = getattr(self, param)
+            except AttributeError:
                 if required:
-                    LOG.exception(exc)
                     raise
                 else:
                     LOG.warning(
@@ -246,40 +259,62 @@ class FileProcessor(object):
                         "but this is not an available parameter.",
                         param,
                     )
-        return arguments
+        return ret
 
-    def check_physical_error(self, error_code, line):
-        """Update attributes based on error code and line."""
-        if error_code == "E101":
-            self.indent_char = line[0]
+    def generate_tokens(self) -> Generator[tokenize.TokenInfo, None, None]:
+        """Tokenize the file and yield the tokens."""
+        for token in tokenize.generate_tokens(self.next_line):
+            if token[2][0] > self.total_lines:
+                break
+            self.tokens.append(token)
+            yield token
 
-    def generate_tokens(self):
-        """Tokenize the file and yield the tokens.
+    def _noqa_line_range(self, min_line: int, max_line: int) -> Dict[int, str]:
+        line_range = range(min_line, max_line + 1)
+        joined = "".join(self.lines[min_line - 1 : max_line])
+        return dict.fromkeys(line_range, joined)
 
-        :raises flake8.exceptions.InvalidSyntax:
-            If a :class:`tokenize.TokenError` is raised while generating
-            tokens.
-        """
-        try:
-            for token in tokenize.generate_tokens(self.next_line):
-                if token[2][0] > self.total_lines:
-                    break
-                self.tokens.append(token)
-                yield token
-        except (tokenize.TokenError, SyntaxError) as exc:
-            raise exceptions.InvalidSyntax(exception=exc)
+    def noqa_line_for(self, line_number: int) -> Optional[str]:
+        """Retrieve the line which will be used to determine noqa."""
+        if self._noqa_line_mapping is None:
+            try:
+                file_tokens = self.file_tokens
+            except (tokenize.TokenError, SyntaxError):
+                # if we failed to parse the file tokens, we'll always fail in
+                # the future, so set this so the code does not try again
+                self._noqa_line_mapping = {}
+            else:
+                ret = {}
+
+                min_line = len(self.lines) + 2
+                max_line = -1
+                for tp, _, (s_line, _), (e_line, _), _ in file_tokens:
+                    if tp == tokenize.ENDMARKER:
+                        break
+
+                    min_line = min(min_line, s_line)
+                    max_line = max(max_line, e_line)
+
+                    if tp in (tokenize.NL, tokenize.NEWLINE):
+                        ret.update(self._noqa_line_range(min_line, max_line))
+
+                        min_line = len(self.lines) + 2
+                        max_line = -1
+
+                # in newer versions of python, a `NEWLINE` token is inserted
+                # at the end of the file even if it doesn't have one.
+                # on old pythons, they will not have hit a `NEWLINE`
+                if max_line != -1:
+                    ret.update(self._noqa_line_range(min_line, max_line))
+
+                self._noqa_line_mapping = ret
 
-    def line_for(self, line_number):
-        """Retrieve the physical line at the specified line number."""
-        adjusted_line_number = line_number - 1
         # NOTE(sigmavirus24): Some plugins choose to report errors for empty
-        # files on Line 1. In those casese, we shouldn't bother trying to
+        # files on Line 1. In those cases, we shouldn't bother trying to
         # retrieve a physical line (since none exist).
-        if 0 <= adjusted_line_number < len(self.lines):
-            return self.lines[adjusted_line_number]
-        return None
+        return self._noqa_line_mapping.get(line_number)
 
-    def next_line(self):
+    def next_line(self) -> str:
         """Get the next line from the list."""
         if self.line_number >= self.total_lines:
             return ""
@@ -289,8 +324,7 @@ class FileProcessor(object):
             self.indent_char = line[0]
         return line
 
-    def read_lines(self):
-        # type: () -> List[str]
+    def read_lines(self) -> List[str]:
         """Read the lines for this file checker."""
         if self.filename is None or self.filename == "-":
             self.filename = self.options.stdin_display_name or "stdin"
@@ -299,13 +333,8 @@ class FileProcessor(object):
             lines = self.read_lines_from_filename()
         return lines
 
-    def _readlines_py2(self):
-        # type: () -> List[str]
-        with open(self.filename, "rU") as fd:
-            return fd.readlines()
-
-    def _readlines_py3(self):
-        # type: () -> List[str]
+    def read_lines_from_filename(self) -> List[str]:
+        """Read the lines for a file."""
         try:
             with tokenize.open(self.filename) as fd:
                 return fd.readlines()
@@ -315,31 +344,20 @@ class FileProcessor(object):
             with open(self.filename, encoding="latin-1") as fd:
                 return fd.readlines()
 
-    def read_lines_from_filename(self):
-        # type: () -> List[str]
-        """Read the lines for a file."""
-        if (2, 6) <= sys.version_info < (3, 0):
-            readlines = self._readlines_py2
-        elif (3, 0) <= sys.version_info < (4, 0):
-            readlines = self._readlines_py3
-        return readlines()
-
-    def read_lines_from_stdin(self):
-        # type: () -> List[str]
+    def read_lines_from_stdin(self) -> List[str]:
         """Read the lines from standard in."""
-        return utils.stdin_get_value().splitlines(True)
+        return utils.stdin_get_lines()
 
-    def should_ignore_file(self):
-        # type: () -> bool
+    def should_ignore_file(self) -> bool:
         """Check if ``flake8: noqa`` is in the file to be ignored.
 
         :returns:
             True if a line matches :attr:`defaults.NOQA_FILE`,
             otherwise False
-        :rtype:
-            bool
         """
-        if any(defaults.NOQA_FILE.match(line) for line in self.lines):
+        if not self.options.disable_noqa and any(
+            defaults.NOQA_FILE.match(line) for line in self.lines
+        ):
             return True
         elif any(defaults.NOQA_FILE.search(line) for line in self.lines):
             LOG.warning(
@@ -350,8 +368,7 @@ class FileProcessor(object):
         else:
             return False
 
-    def strip_utf_bom(self):
-        # type: () -> NoneType
+    def strip_utf_bom(self) -> None:
         """Strip the UTF bom from the lines of the file."""
         if not self.lines:
             # If we have nothing to analyze quit early
@@ -368,61 +385,31 @@ class FileProcessor(object):
             self.lines[0] = self.lines[0][3:]
 
 
-def is_eol_token(token):
+def is_eol_token(token: tokenize.TokenInfo) -> bool:
     """Check if the token is an end-of-line token."""
     return token[0] in NEWLINE or token[4][token[3][1] :].lstrip() == "\\\n"
 
 
-if COMMENT_WITH_NL:  # If on Python 2.6
-
-    def is_eol_token(token, _is_eol_token=is_eol_token):
-        """Check if the token is an end-of-line token."""
-        return _is_eol_token(token) or (
-            token[0] == tokenize.COMMENT and token[1] == token[4]
-        )
-
-
-def is_multiline_string(token):
+def is_multiline_string(token: tokenize.TokenInfo) -> bool:
     """Check if this is a multiline string."""
     return token[0] == tokenize.STRING and "\n" in token[1]
 
 
-def token_is_newline(token):
+def token_is_newline(token: tokenize.TokenInfo) -> bool:
     """Check if the token type is a newline token type."""
     return token[0] in NEWLINE
 
 
-def token_is_comment(token):
-    """Check if the token type is a comment."""
-    return COMMENT_WITH_NL and token[0] == tokenize.COMMENT
-
-
-def count_parentheses(current_parentheses_count, token_text):
+def count_parentheses(current_parentheses_count: int, token_text: str) -> int:
     """Count the number of parentheses."""
-    current_parentheses_count = current_parentheses_count or 0
-    if token_text in "([{":
+    if token_text in "([{":  # nosec
         return current_parentheses_count + 1
-    elif token_text in "}])":
+    elif token_text in "}])":  # nosec
         return current_parentheses_count - 1
     return current_parentheses_count
 
 
-def log_token(log, token):
-    """Log a token to a provided logging object."""
-    if token[2][0] == token[3][0]:
-        pos = "[%s:%s]" % (token[2][1] or "", token[3][1])
-    else:
-        pos = "l.%s" % token[3][0]
-    log.log(
-        flake8._EXTRA_VERBOSE,
-        "l.%s\t%s\t%s\t%r"
-        % (token[2][0], pos, tokenize.tok_name[token[0]], token[1]),
-    )
-
-
-# NOTE(sigmavirus24): This was taken wholesale from
-# https://github.com/PyCQA/pycodestyle
-def expand_indent(line):
+def expand_indent(line: str) -> int:
     r"""Return the amount of indentation.
 
     Tabs are expanded to the next multiple of 8.
@@ -436,30 +423,20 @@ def expand_indent(line):
     >>> expand_indent('        \t')
     16
     """
-    if "\t" not in line:
-        return len(line) - len(line.lstrip())
-    result = 0
-    for char in line:
-        if char == "\t":
-            result = result // 8 * 8 + 8
-        elif char == " ":
-            result += 1
-        else:
-            break
-    return result
+    return len(line.expandtabs(8))
 
 
 # NOTE(sigmavirus24): This was taken wholesale from
 # https://github.com/PyCQA/pycodestyle. The in-line comments were edited to be
 # more descriptive.
-def mutate_string(text):
+def mutate_string(text: str) -> str:
     """Replace contents with 'xxx' to prevent syntax matching.
 
-    >>> mute_string('"abc"')
+    >>> mutate_string('"abc"')
     '"xxx"'
-    >>> mute_string("'''abc'''")
+    >>> mutate_string("'''abc'''")
     "'''xxx'''"
-    >>> mute_string("r'abc'")
+    >>> mutate_string("r'abc'")
     "r'xxx'"
     """
     # NOTE(sigmavirus24): If there are string modifiers (e.g., b, u, r)
