@@ -1,17 +1,14 @@
 """Implementation of the StyleGuide used by Flake8."""
 import argparse
-import collections
 import contextlib
 import copy
 import enum
 import functools
 import itertools
-import linecache
 import logging
 from typing import Dict
 from typing import Generator
 from typing import List
-from typing import Match
 from typing import Optional
 from typing import Sequence
 from typing import Set
@@ -22,6 +19,7 @@ from flake8 import defaults
 from flake8 import statistics
 from flake8 import utils
 from flake8.formatting import base as base_formatter
+from flake8.violation import Violation
 
 __all__ = ("StyleGuide",)
 
@@ -49,101 +47,28 @@ class Decision(enum.Enum):
     Selected = "selected error"
 
 
-@functools.lru_cache(maxsize=512)
-def find_noqa(physical_line: str) -> Optional[Match[str]]:
-    return defaults.NOQA_INLINE_REGEXP.search(physical_line)
+def _explicitly_chosen(
+    *,
+    option: Optional[List[str]],
+    extend: Optional[List[str]],
+) -> Tuple[str, ...]:
+    ret = [*(option or []), *(extend or [])]
+    return tuple(sorted(ret, reverse=True))
 
 
-class Violation(
-    collections.namedtuple(
-        "Violation",
-        [
-            "code",
-            "filename",
-            "line_number",
-            "column_number",
-            "text",
-            "physical_line",
-        ],
-    )
-):
-    """Class representing a violation reported by Flake8."""
-
-    def is_inline_ignored(self, disable_noqa: bool) -> bool:
-        """Determine if a comment has been added to ignore this line.
-
-        :param bool disable_noqa:
-            Whether or not users have provided ``--disable-noqa``.
-        :returns:
-            True if error is ignored in-line, False otherwise.
-        :rtype:
-            bool
-        """
-        physical_line = self.physical_line
-        # TODO(sigmavirus24): Determine how to handle stdin with linecache
-        if disable_noqa:
-            return False
-
-        if physical_line is None:
-            physical_line = linecache.getline(self.filename, self.line_number)
-        noqa_match = find_noqa(physical_line)
-        if noqa_match is None:
-            LOG.debug("%r is not inline ignored", self)
-            return False
-
-        codes_str = noqa_match.groupdict()["codes"]
-        if codes_str is None:
-            LOG.debug("%r is ignored by a blanket ``# noqa``", self)
-            return True
-
-        codes = set(utils.parse_comma_separated_list(codes_str))
-        if self.code in codes or self.code.startswith(tuple(codes)):
-            LOG.debug(
-                "%r is ignored specifically inline with ``# noqa: %s``",
-                self,
-                codes_str,
-            )
-            return True
-
-        LOG.debug(
-            "%r is not ignored inline with ``# noqa: %s``", self, codes_str
-        )
-        return False
-
-    def is_in(self, diff: Dict[str, Set[int]]) -> bool:
-        """Determine if the violation is included in a diff's line ranges.
-
-        This function relies on the parsed data added via
-        :meth:`~StyleGuide.add_diff_ranges`. If that has not been called and
-        we are not evaluating files in a diff, then this will always return
-        True. If there are diff ranges, then this will return True if the
-        line number in the error falls inside one of the ranges for the file
-        (and assuming the file is part of the diff data). If there are diff
-        ranges, this will return False if the file is not part of the diff
-        data or the line number of the error is not in any of the ranges of
-        the diff.
-
-        :returns:
-            True if there is no diff or if the error is in the diff's line
-            number ranges. False if the error's line number falls outside
-            the diff's line number ranges.
-        :rtype:
-            bool
-        """
-        if not diff:
-            return True
-
-        # NOTE(sigmavirus24): The parsed diff will be a defaultdict with
-        # a set as the default value (if we have received it from
-        # flake8.utils.parse_unified_diff). In that case ranges below
-        # could be an empty set (which is False-y) or if someone else
-        # is using this API, it could be None. If we could guarantee one
-        # or the other, we would check for it more explicitly.
-        line_numbers = diff.get(self.filename)
-        if not line_numbers:
-            return False
-
-        return self.line_number in line_numbers
+def _select_ignore(
+    *,
+    option: Optional[List[str]],
+    default: Tuple[str, ...],
+    extended_default: List[str],
+    extend: Optional[List[str]],
+) -> Tuple[str, ...]:
+    # option was explicitly set, ignore the default and extended default
+    if option is not None:
+        ret = [*option, *(extend or [])]
+    else:
+        ret = [*default, *extended_default, *(extend or [])]
+    return tuple(sorted(ret, reverse=True))
 
 
 class DecisionEngine:
@@ -156,45 +81,33 @@ class DecisionEngine:
     def __init__(self, options: argparse.Namespace) -> None:
         """Initialize the engine."""
         self.cache: Dict[str, Decision] = {}
-        self.selected = tuple(options.select)
-        self.extended_selected = tuple(
-            sorted(options.extended_default_select, reverse=True)
-        )
-        self.enabled_extensions = tuple(options.enable_extensions)
-        self.all_selected = tuple(
-            sorted(
-                itertools.chain(
-                    self.selected,
-                    options.extend_select,
-                    self.enabled_extensions,
-                ),
-                reverse=True,
-            )
-        )
-        self.ignored = tuple(
-            sorted(
-                itertools.chain(options.ignore, options.extend_ignore),
-                reverse=True,
-            )
-        )
-        self.using_default_ignore = set(self.ignored) == set(
-            defaults.IGNORE
-        ).union(options.extended_default_ignore)
-        self.using_default_select = set(self.selected) == set(defaults.SELECT)
 
-    def _in_all_selected(self, code: str) -> bool:
-        return bool(self.all_selected) and code.startswith(self.all_selected)
+        self.selected_explicitly = _explicitly_chosen(
+            option=options.select,
+            extend=options.extend_select,
+        )
+        self.ignored_explicitly = _explicitly_chosen(
+            option=options.ignore,
+            extend=options.extend_ignore,
+        )
 
-    def _in_extended_selected(self, code: str) -> bool:
-        return bool(self.extended_selected) and code.startswith(
-            self.extended_selected
+        self.selected = _select_ignore(
+            option=options.select,
+            default=defaults.SELECT,
+            extended_default=options.extended_default_select,
+            extend=options.extend_select,
+        )
+        self.ignored = _select_ignore(
+            option=options.ignore,
+            default=defaults.IGNORE,
+            extended_default=options.extended_default_ignore,
+            extend=options.extend_ignore,
         )
 
     def was_selected(self, code: str) -> Union[Selected, Ignored]:
         """Determine if the code has been selected by the user.
 
-        :param str code:
-            The code for the check that has been run.
+        :param code: The code for the check that has been run.
         :returns:
             Selected.Implicitly if the selected list is empty,
             Selected.Explicitly if the selected list is not empty and a match
@@ -202,21 +115,17 @@ class DecisionEngine:
             Ignored.Implicitly if the selected list is not empty but no match
             was found.
         """
-        if self._in_all_selected(code):
+        if code.startswith(self.selected_explicitly):
             return Selected.Explicitly
-
-        if not self.all_selected and self._in_extended_selected(code):
-            # If it was not explicitly selected, it may have been implicitly
-            # selected because the check comes from a plugin that is enabled by
-            # default
+        elif code.startswith(self.selected):
             return Selected.Implicitly
-
-        return Ignored.Implicitly
+        else:
+            return Ignored.Implicitly
 
     def was_ignored(self, code: str) -> Union[Selected, Ignored]:
         """Determine if the code has been ignored by the user.
 
-        :param str code:
+        :param code:
             The code for the check that has been run.
         :returns:
             Selected.Implicitly if the ignored list is empty,
@@ -225,84 +134,54 @@ class DecisionEngine:
             Selected.Implicitly if the ignored list is not empty but no match
             was found.
         """
-        if self.ignored and code.startswith(self.ignored):
+        if code.startswith(self.ignored_explicitly):
             return Ignored.Explicitly
-
-        return Selected.Implicitly
-
-    def more_specific_decision_for(self, code: str) -> Decision:
-        select = find_first_match(code, self.all_selected)
-        extra_select = find_first_match(code, self.extended_selected)
-        ignore = find_first_match(code, self.ignored)
-
-        if select and ignore:
-            # If the violation code appears in both the select and ignore
-            # lists (in some fashion) then if we're using the default ignore
-            # list and a custom select list we should select the code. An
-            # example usage looks like this:
-            #   A user has a code that would generate an E126 violation which
-            #   is in our default ignore list and they specify select=E.
-            # We should be reporting that violation. This logic changes,
-            # however, if they specify select and ignore such that both match.
-            # In that case we fall through to our find_more_specific call.
-            # If, however, the user hasn't specified a custom select, and
-            # we're using the defaults for both select and ignore then the
-            # more specific rule must win. In most cases, that will be to
-            # ignore the violation since our default select list is very
-            # high-level and our ignore list is highly specific.
-            if self.using_default_ignore and not self.using_default_select:
-                return Decision.Selected
-            return find_more_specific(select, ignore)
-        if extra_select and ignore:
-            # At this point, select is false-y. Now we need to check if the
-            # code is in our extended select list and our ignore list. This is
-            # a *rare* case as we see little usage of the extended select list
-            # that plugins can use, so I suspect this section may change to
-            # look a little like the block above in which we check if we're
-            # using our default ignore list.
-            return find_more_specific(extra_select, ignore)
-        if select or (extra_select and self.using_default_select):
-            # Here, ignore was false-y and the user has either selected
-            # explicitly the violation or the violation is covered by
-            # something in the extended select list and we're using the
-            # default select list. In either case, we want the violation to be
-            # selected.
-            return Decision.Selected
-        if select is None and (
-            extra_select is None or not self.using_default_ignore
-        ):
-            return Decision.Ignored
-        if (select is None and not self.using_default_select) and (
-            ignore is None and self.using_default_ignore
-        ):
-            return Decision.Ignored
-        return Decision.Selected
+        elif code.startswith(self.ignored):
+            return Ignored.Implicitly
+        else:
+            return Selected.Implicitly
 
     def make_decision(self, code: str) -> Decision:
         """Decide if code should be ignored or selected."""
-        LOG.debug('Deciding if "%s" should be reported', code)
         selected = self.was_selected(code)
         ignored = self.was_ignored(code)
         LOG.debug(
-            'The user configured "%s" to be "%s", "%s"',
+            "The user configured %r to be %r, %r",
             code,
             selected,
             ignored,
         )
 
-        if (
-            selected is Selected.Explicitly or selected is Selected.Implicitly
-        ) and ignored is Selected.Implicitly:
-            decision = Decision.Selected
+        if isinstance(selected, Selected) and isinstance(ignored, Selected):
+            return Decision.Selected
+        elif isinstance(selected, Ignored) and isinstance(ignored, Ignored):
+            return Decision.Ignored
+        elif (
+            selected is Selected.Explicitly
+            and ignored is not Ignored.Explicitly
+        ):
+            return Decision.Selected
+        elif (
+            selected is not Selected.Explicitly
+            and ignored is Ignored.Explicitly
+        ):
+            return Decision.Ignored
+        elif selected is Ignored.Implicitly and ignored is Selected.Implicitly:
+            return Decision.Ignored
         elif (
             selected is Selected.Explicitly and ignored is Ignored.Explicitly
         ) or (
-            selected is Ignored.Implicitly and ignored is Selected.Implicitly
+            selected is Selected.Implicitly and ignored is Ignored.Implicitly
         ):
-            decision = self.more_specific_decision_for(code)
-        elif selected is Ignored.Implicitly or ignored is Ignored.Explicitly:
-            decision = Decision.Ignored  # pylint: disable=R0204
-        return decision
+            # we only get here if it was in both lists: longest prefix wins
+            select = next(s for s in self.selected if code.startswith(s))
+            ignore = next(s for s in self.ignored if code.startswith(s))
+            if len(select) > len(ignore):
+                return Decision.Selected
+            else:
+                return Decision.Ignored
+        else:
+            raise AssertionError(f"unreachable {code} {selected} {ignored}")
 
     def decision_for(self, code: str) -> Decision:
         """Return the decision for a specific code.
@@ -315,8 +194,7 @@ class DecisionEngine:
         This method does not look at whether the specific line is being
         ignored in the file itself.
 
-        :param str code:
-            The code for the check that has been run.
+        :param code: The code for the check that has been run.
         """
         decision = self.cache.get(code)
         if decision is None:
@@ -354,6 +232,10 @@ class StyleGuideManager:
             )
         )
 
+        self.style_guide_for = functools.lru_cache(maxsize=None)(
+            self._style_guide_for
+        )
+
     def populate_style_guides_with(
         self, options: argparse.Namespace
     ) -> Generator["StyleGuide", None, None]:
@@ -361,12 +243,8 @@ class StyleGuideManager:
 
         :param options:
             The original options parsed from the CLI and config file.
-        :type options:
-            :class:`~argparse.Namespace`
         :returns:
             A copy of the default style guide with overridden values.
-        :rtype:
-            :class:`~flake8.style_guide.StyleGuide`
         """
         per_file = utils.parse_files_to_codes_mapping(options.per_file_ignores)
         for filename, violations in per_file:
@@ -374,16 +252,12 @@ class StyleGuideManager:
                 filename=filename, extend_ignore_with=violations
             )
 
-    @functools.lru_cache(maxsize=None)
-    def style_guide_for(self, filename: str) -> "StyleGuide":
+    def _style_guide_for(self, filename: str) -> "StyleGuide":
         """Find the StyleGuide for the filename in particular."""
-        guides = sorted(
+        return max(
             (g for g in self.style_guides if g.applies_to(filename)),
             key=lambda g: len(g.filename or ""),
         )
-        if len(guides) > 1:
-            return guides[-1]
-        return guides[0]
 
     @contextlib.contextmanager
     def processing_file(
@@ -399,31 +273,29 @@ class StyleGuideManager:
         code: str,
         filename: str,
         line_number: int,
-        column_number: Optional[int],
+        column_number: int,
         text: str,
         physical_line: Optional[str] = None,
     ) -> int:
         """Handle an error reported by a check.
 
-        :param str code:
+        :param code:
             The error code found, e.g., E123.
-        :param str filename:
+        :param filename:
             The file in which the error was found.
-        :param int line_number:
+        :param line_number:
             The line number (where counting starts at 1) at which the error
             occurs.
-        :param int column_number:
+        :param column_number:
             The column number (where counting starts at 1) at which the error
             occurs.
-        :param str text:
+        :param text:
             The text of the error message.
-        :param str physical_line:
+        :param physical_line:
             The actual physical line causing the error.
         :returns:
             1 if the error was reported. 0 if it was ignored. This is to allow
             for counting of the number of errors found that were not ignored.
-        :rtype:
-            int
         """
         guide = self.style_guide_for(filename)
         return guide.handle_error(
@@ -436,7 +308,7 @@ class StyleGuideManager:
         This provides information to the underlying StyleGuides so that only
         the errors in the line number ranges are reported.
 
-        :param dict diffinfo:
+        :param diffinfo:
             Dictionary mapping filenames to sets of line number ranges.
         """
         for guide in self.style_guides:
@@ -479,7 +351,8 @@ class StyleGuide:
         """Create a copy of this style guide with different values."""
         filename = filename or self.filename
         options = copy.deepcopy(self.options)
-        options.ignore.extend(extend_ignore_with or [])
+        options.extend_ignore = options.extend_ignore or []
+        options.extend_ignore.extend(extend_ignore_with or [])
         return StyleGuide(
             options, self.formatter, self.stats, filename=filename
         )
@@ -496,13 +369,11 @@ class StyleGuide:
     def applies_to(self, filename: str) -> bool:
         """Check if this StyleGuide applies to the file.
 
-        :param str filename:
+        :param filename:
             The name of the file with violations that we're potentially
             applying this StyleGuide to.
         :returns:
             True if this applies, False otherwise
-        :rtype:
-            bool
         """
         if self.filename is None:
             return True
@@ -522,7 +393,7 @@ class StyleGuide:
         This method does not look at whether the specific line is being
         ignored in the file itself.
 
-        :param str code:
+        :param code:
             The code for the check that has been run.
         """
         return self.decider.decision_for(code)
@@ -532,36 +403,33 @@ class StyleGuide:
         code: str,
         filename: str,
         line_number: int,
-        column_number: Optional[int],
+        column_number: int,
         text: str,
         physical_line: Optional[str] = None,
     ) -> int:
         """Handle an error reported by a check.
 
-        :param str code:
+        :param code:
             The error code found, e.g., E123.
-        :param str filename:
+        :param filename:
             The file in which the error was found.
-        :param int line_number:
+        :param line_number:
             The line number (where counting starts at 1) at which the error
             occurs.
-        :param int column_number:
+        :param column_number:
             The column number (where counting starts at 1) at which the error
             occurs.
-        :param str text:
+        :param text:
             The text of the error message.
-        :param str physical_line:
+        :param physical_line:
             The actual physical line causing the error.
         :returns:
             1 if the error was reported. 0 if it was ignored. This is to allow
             for counting of the number of errors found that were not ignored.
-        :rtype:
-            int
         """
         disable_noqa = self.options.disable_noqa
         # NOTE(sigmavirus24): Apparently we're provided with 0-indexed column
-        # numbers so we have to offset that here. Also, if a SyntaxError is
-        # caught, column_number may be None.
+        # numbers so we have to offset that here.
         if not column_number:
             column_number = 0
         error = Violation(
@@ -589,25 +457,7 @@ class StyleGuide:
         This provides information to the StyleGuide so that only the errors
         in the line number ranges are reported.
 
-        :param dict diffinfo:
+        :param diffinfo:
             Dictionary mapping filenames to sets of line number ranges.
         """
         self._parsed_diff = diffinfo
-
-
-def find_more_specific(selected: str, ignored: str) -> Decision:
-    if selected.startswith(ignored) and selected != ignored:
-        return Decision.Selected
-    return Decision.Ignored
-
-
-def find_first_match(
-    error_code: str, code_list: Tuple[str, ...]
-) -> Optional[str]:
-    startswith = error_code.startswith
-    for code in code_list:
-        if startswith(code):
-            break
-    else:
-        return None
-    return code
