@@ -1,11 +1,13 @@
 """Tests for the Manager object for FileCheckers."""
 import errno
+import multiprocessing
 from unittest import mock
 
 import pytest
 
 from flake8 import checker
 from flake8.main.options import JobsArgument
+from flake8.plugins import finder
 
 
 def style_guide_mock():
@@ -21,7 +23,7 @@ def style_guide_mock():
 def _parallel_checker_manager():
     """Call Manager.run() and return the number of calls to `run_serial`."""
     style_guide = style_guide_mock()
-    manager = checker.Manager(style_guide, [], [])
+    manager = checker.Manager(style_guide, finder.Checkers([], [], []))
     # multiple checkers is needed for parallel mode
     manager.checkers = [mock.Mock(), mock.Mock()]
     return manager
@@ -37,8 +39,8 @@ def test_oserrors_cause_serial_fall_back():
     assert serial.call_count == 1
 
 
-@mock.patch("flake8.checker._multiprocessing_is_fork", return_value=True)
-def test_oserrors_are_reraised(is_windows):
+@mock.patch.object(multiprocessing, "get_start_method", return_value="fork")
+def test_oserrors_are_reraised(_):
     """Verify that unexpected OSErrors will cause the Manager to reraise."""
     err = OSError(errno.EAGAIN, "Ominous message")
     with mock.patch("_multiprocessing.SemLock", side_effect=err):
@@ -49,34 +51,40 @@ def test_oserrors_are_reraised(is_windows):
     assert serial.call_count == 0
 
 
-def test_multiprocessing_is_disabled():
+@mock.patch.object(multiprocessing, "get_start_method", return_value="spawn")
+def test_multiprocessing_is_disabled(_):
     """Verify not being able to import multiprocessing forces jobs to 0."""
     style_guide = style_guide_mock()
-    with mock.patch("flake8.checker.multiprocessing", None):
-        manager = checker.Manager(style_guide, [], [])
-        assert manager.jobs == 0
+    manager = checker.Manager(style_guide, finder.Checkers([], [], []))
+    assert manager.jobs == 0
 
 
-def test_make_checkers():
+def test_multiprocessing_cpu_count_not_implemented():
+    """Verify that jobs is 0 if cpu_count is unavailable."""
+    style_guide = style_guide_mock()
+    style_guide.options.jobs = JobsArgument("auto")
+
+    with mock.patch.object(
+        multiprocessing,
+        "cpu_count",
+        side_effect=NotImplementedError,
+    ):
+        manager = checker.Manager(style_guide, finder.Checkers([], [], []))
+    assert manager.jobs == 0
+
+
+@mock.patch.object(multiprocessing, "get_start_method", return_value="spawn")
+def test_make_checkers(_):
     """Verify that we create a list of FileChecker instances."""
     style_guide = style_guide_mock()
-    files = ["file1", "file2"]
-    checkplugins = mock.Mock()
-    checkplugins.to_dictionary.return_value = {
-        "ast_plugins": [],
-        "logical_line_plugins": [],
-        "physical_line_plugins": [],
-    }
-    with mock.patch("flake8.checker.multiprocessing", None):
-        manager = checker.Manager(style_guide, files, checkplugins)
+    style_guide.options.filenames = ["file1", "file2"]
+    manager = checker.Manager(style_guide, finder.Checkers([], [], []))
 
-    with mock.patch("flake8.utils.filenames_from") as filenames_from:
-        filenames_from.side_effect = [["file1"], ["file2"]]
-        with mock.patch("flake8.utils.fnmatch", return_value=True):
-            with mock.patch("flake8.processor.FileProcessor"):
-                manager.make_checkers()
+    with mock.patch("flake8.utils.fnmatch", return_value=True):
+        with mock.patch("flake8.processor.FileProcessor"):
+            manager.make_checkers(["file1", "file2"])
 
     assert manager._all_checkers
     for file_checker in manager._all_checkers:
-        assert file_checker.filename in files
+        assert file_checker.filename in style_guide.options.filenames
     assert not manager.checkers  # the files don't exist
