@@ -14,11 +14,12 @@ from flake8 import exceptions
             exception=ValueError("boom!"),
         ),
         exceptions.PluginRequestedUnknownParameters(
-            plugin={"plugin_name": "plugin_name"},
+            plugin_name="plugin_name",
             exception=ValueError("boom!"),
         ),
         exceptions.PluginExecutionFailed(
-            plugin={"plugin_name": "plugin_name"},
+            filename="filename.py",
+            plugin_name="plugin_name",
             exception=ValueError("boom!"),
         ),
     ),
