@@ -209,21 +209,21 @@ def test_next_line(default_options):
     [
         (
             {"blank_before": True, "blank_lines": True},
-            None,
+            {},
             {"blank_before": 0, "blank_lines": 0},
         ),
         (
             {"noqa": True, "fake": True},
             {"fake": "foo"},
-            {"noqa": False, "fake": "foo"},
+            {"noqa": False},
         ),
         (
             {"blank_before": True, "blank_lines": True, "noqa": True},
             {"blank_before": 10, "blank_lines": 5, "noqa": True},
-            {"blank_before": 10, "blank_lines": 5, "noqa": True},
+            {},
         ),
-        ({}, {"fake": "foo"}, {"fake": "foo"}),
-        ({"non-existent": False}, {"fake": "foo"}, {"fake": "foo"}),
+        ({}, {"fake": "foo"}, {}),
+        ({"non-existent": False}, {"fake": "foo"}, {}),
     ],
 )
 def test_keyword_arguments_for(params, args, expected_kwargs, default_options):
@@ -235,9 +235,9 @@ def test_keyword_arguments_for(params, args, expected_kwargs, default_options):
             "Line 1",
         ],
     )
-    kwargs_for = file_processor.keyword_arguments_for
+    ret = file_processor.keyword_arguments_for(params, args)
 
-    assert kwargs_for(params, args) == expected_kwargs
+    assert ret == expected_kwargs
 
 
 def test_keyword_arguments_for_does_not_handle_attribute_errors(
@@ -253,33 +253,33 @@ def test_keyword_arguments_for_does_not_handle_attribute_errors(
     )
 
     with pytest.raises(AttributeError):
-        file_processor.keyword_arguments_for({"fake": True})
+        file_processor.keyword_arguments_for({"fake": True}, {})
 
 
-@pytest.mark.parametrize(
-    "unsplit_line, expected_lines",
-    [
-        ("line", []),
-        ("line 1\n", ["line 1"]),
-        ("line 1\nline 2\n", ["line 1", "line 2"]),
-        ("line 1\n\nline 2\n", ["line 1", "", "line 2"]),
-    ],
-)
-def test_split_line(unsplit_line, expected_lines, default_options):
-    """Verify the token line splitting."""
+def test_processor_split_line(default_options):
     file_processor = processor.FileProcessor(
         "-",
         default_options,
         lines=[
-            "Line 1",
+            'x = """\n',
+            "contents\n",
+            '"""\n',
         ],
     )
-
-    token = (1, unsplit_line, (0, 0), (0, 0), "")
-    actual_lines = list(file_processor.split_line(token))
-    assert expected_lines == actual_lines
-
-    assert len(actual_lines) == file_processor.line_number
+    token = tokenize.TokenInfo(
+        3,
+        '"""\ncontents\n"""',
+        (1, 4),
+        (3, 3),
+        'x = """\ncontents\n"""\n',
+    )
+    expected = [('x = """\n', 0), ("contents\n", 1)]
+    actual = [
+        (line, file_processor.line_number)
+        for line in file_processor.split_line(token)
+    ]
+    assert expected == actual
+    assert file_processor.line_number == 2
 
 
 def test_build_ast(default_options):
@@ -380,51 +380,6 @@ def test_expand_indent(string, expected):
     assert expected == actual
 
 
-@pytest.mark.parametrize(
-    "token, log_string",
-    [
-        [
-            (
-                tokenize.COMMENT,
-                "# this is a comment",
-                (1, 0),  # (start_row, start_column)
-                (1, 19),  # (end_ro, end_column)
-                "# this is a comment",
-            ),
-            "l.1\t[:19]\tCOMMENT\t'# this is a comment'",
-        ],
-        [
-            (
-                tokenize.COMMENT,
-                "# this is a comment",
-                (1, 5),  # (start_row, start_column)
-                (1, 19),  # (end_ro, end_column)
-                "# this is a comment",
-            ),
-            "l.1\t[5:19]\tCOMMENT\t'# this is a comment'",
-        ],
-        [
-            (
-                tokenize.COMMENT,
-                "# this is a comment",
-                (1, 0),  # (start_row, start_column)
-                (2, 19),  # (end_ro, end_column)
-                "# this is a comment",
-            ),
-            "l.1\tl.2\tCOMMENT\t'# this is a comment'",
-        ],
-    ],
-)
-def test_log_token(token, log_string):
-    """Verify we use the log object passed in."""
-    log = mock.Mock()
-    processor.log_token(log, token)
-    log.log.assert_called_once_with(
-        5,  # flake8._EXTRA_VERBOSE
-        log_string,
-    )
-
-
 @pytest.mark.parametrize(
     "current_count, token_text, expected",
     [
