@@ -1,5 +1,6 @@
 """Tests for Flake8's legacy API."""
 import argparse
+import configparser
 import os.path
 from unittest import mock
 
@@ -7,7 +8,7 @@ import pytest
 
 from flake8.api import legacy as api
 from flake8.formatting import base as formatter
-from flake8.options.config import ConfigFileFinder
+from flake8.options import config
 
 
 def test_get_style_guide():
@@ -18,26 +19,32 @@ def test_get_style_guide():
         isolated=False,
         output_file=None,
         verbose=0,
+        enable_extensions=None,
+        require_plugins=None,
     )
     mockedapp = mock.Mock()
     mockedapp.parse_preliminary_options.return_value = (prelim_opts, [])
     mockedapp.program = "flake8"
-    with mock.patch(
-        "flake8.api.legacy.config.ConfigFileFinder"
-    ) as mock_config_finder:  # noqa: E501
-        config_finder = ConfigFileFinder(mockedapp.program)
-        mock_config_finder.return_value = config_finder
 
+    cfg = configparser.RawConfigParser()
+    cfg_dir = os.getcwd()
+
+    with mock.patch.object(config, "load_config", return_value=(cfg, cfg_dir)):
         with mock.patch("flake8.main.application.Application") as application:
             application.return_value = mockedapp
             style_guide = api.get_style_guide()
 
     application.assert_called_once_with()
     mockedapp.parse_preliminary_options.assert_called_once_with([])
-    mockedapp.find_plugins.assert_called_once_with(config_finder)
+    mockedapp.find_plugins.assert_called_once_with(
+        cfg,
+        cfg_dir,
+        enable_extensions=None,
+        require_plugins=None,
+    )
     mockedapp.register_plugin_options.assert_called_once_with()
     mockedapp.parse_configuration_and_cli.assert_called_once_with(
-        config_finder, []
+        cfg, cfg_dir, []
     )
     mockedapp.make_formatter.assert_called_once_with()
     mockedapp.make_guide.assert_called_once_with()
@@ -56,9 +63,9 @@ def test_styleguide_options():
 def test_styleguide_paths():
     """Show that we proxy the StyleGuide.paths attribute."""
     app = mock.Mock()
-    app.paths = "paths"
+    app.options.filenames = ["paths"]
     style_guide = api.StyleGuide(app)
-    assert style_guide.paths == "paths"
+    assert style_guide.paths == ["paths"]
 
 
 def test_styleguide_check_files():
@@ -68,7 +75,8 @@ def test_styleguide_check_files():
     style_guide = api.StyleGuide(app)
     report = style_guide.check_files(paths)
 
-    app.run_checks.assert_called_once_with(paths)
+    assert app.options.filenames == paths
+    app.run_checks.assert_called_once_with()
     app.report_errors.assert_called_once_with()
     assert isinstance(report, api.Report)
 
@@ -76,33 +84,13 @@ def test_styleguide_check_files():
 def test_styleguide_excluded():
     """Verify we delegate to our file checker manager.
 
-    We also want to ensure that if we don't specify a parent, is_path_excluded
-    is called exactly once.
-    """
-    app = mock.Mock()
-    file_checker_manager = app.file_checker_manager = mock.Mock()
-    style_guide = api.StyleGuide(app)
-
-    style_guide.excluded("file.py")
-    file_checker_manager.is_path_excluded.assert_called_once_with("file.py")
-
-
-def test_styleguide_excluded_with_parent():
-    """Verify we delegate to our file checker manager.
-
     When we add the parent argument, we don't check that is_path_excluded was
     called only once.
     """
-    app = mock.Mock()
-    file_checker_manager = app.file_checker_manager = mock.Mock()
-    file_checker_manager.is_path_excluded.return_value = False
-    style_guide = api.StyleGuide(app)
-
-    style_guide.excluded("file.py", "parent")
-    assert file_checker_manager.is_path_excluded.call_args_list == [
-        mock.call("file.py"),
-        mock.call(os.path.join("parent", "file.py")),
-    ]
+    style_guide = api.get_style_guide(exclude=["file*", "*/parent/*"])
+    assert not style_guide.excluded("unrelated.py")
+    assert style_guide.excluded("file.py")
+    assert style_guide.excluded("test.py", "parent")
 
 
 def test_styleguide_init_report_does_nothing():
@@ -119,7 +107,7 @@ def test_styleguide_init_report_with_non_subclass():
     app = mock.Mock()
     style_guide = api.StyleGuide(app)
     with pytest.raises(ValueError):
-        style_guide.init_report(object)
+        style_guide.init_report(object)  # type: ignore
     assert app.make_formatter.called is False
     assert app.make_guide.called is False
 
@@ -134,7 +122,7 @@ def test_styleguide_init_report():
             raise NotImplementedError
 
     style_guide.init_report(FakeFormatter)
-    app.make_formatter.assert_called_once_with(FakeFormatter)
+    assert isinstance(app.formatter, FakeFormatter)
     assert app.guide is None
     app.make_guide.assert_called_once_with()
 
