@@ -1,12 +1,13 @@
 """Tests for the Nothing formatter obbject."""
 import argparse
 
-from flake8 import style_guide
 from flake8.formatting import default
+from flake8.violation import Violation
 
 
 def options(**kwargs):
     """Create an argparse.Namespace instance."""
+    kwargs.setdefault("color", "auto")
     kwargs.setdefault("output_file", None)
     kwargs.setdefault("tee", False)
     return argparse.Namespace(**kwargs)
@@ -15,7 +16,7 @@ def options(**kwargs):
 def test_format_returns_nothing():
     """Verify Nothing.format returns None."""
     formatter = default.Nothing(options())
-    error = style_guide.Violation("code", "file.py", 1, 1, "text", "1")
+    error = Violation("code", "file.py", 1, 1, "text", "1")
 
     assert formatter.format(error) is None
 
@@ -23,6 +24,6 @@ def test_format_returns_nothing():
 def test_show_source_returns_nothing():
     """Verify Nothing.show_source returns None."""
     formatter = default.Nothing(options())
-    error = style_guide.Violation("code", "file.py", 1, 1, "text", "1")
+    error = Violation("code", "file.py", 1, 1, "text", "1")
 
     assert formatter.show_source(error) is None
