@@ -25,7 +25,7 @@ def f():
     sys = sys
 """
     tree = ast.parse(src)
-    checker = pyflakes_shim.FlakesChecker(tree, (), "t.py")
+    checker = pyflakes_shim.FlakesChecker(tree, [], "t.py")
     message_texts = [s for _, _, s, _ in checker.run()]
     assert message_texts == [
         "F823 local variable 'sys' defined in enclosing scope on line 1 referenced before assignment",  # noqa: E501
