@@ -9,7 +9,6 @@ import pytest
 
 from flake8 import exceptions
 from flake8 import utils
-from flake8.plugins import manager as plugin_manager
 
 RELATIVE_PATHS = ["flake8", "pep8", "pyflakes", "mccabe"]
 
@@ -133,6 +132,7 @@ def test_invalid_file_list(value):
     "value,expected",
     [
         ("flake8", "flake8"),
+        (".", os.path.abspath(".")),
         ("../flake8", os.path.abspath("../flake8")),
         ("flake8/", os.path.abspath("flake8")),
     ],
@@ -160,13 +160,11 @@ def test_normalize_paths(value, expected):
     assert utils.normalize_paths(value) == expected
 
 
-def test_is_windows_checks_for_nt():
-    """Verify that we correctly detect Windows."""
-    with mock.patch.object(os, "name", "nt"):
-        assert utils.is_windows() is True
-
-    with mock.patch.object(os, "name", "posix"):
-        assert utils.is_windows() is False
+def test_matches_filename_for_excluding_dotfiles():
+    """Verify that `.` and `..` are not matched by `.*`."""
+    logger = logging.Logger(__name__)
+    assert not utils.matches_filename(".", (".*",), "", logger)
+    assert not utils.matches_filename("..", (".*",), "", logger)
 
 
 @pytest.mark.parametrize(
@@ -183,117 +181,6 @@ def test_fnmatch(filename, patterns, expected):
     assert utils.fnmatch(filename, patterns) is expected
 
 
-@pytest.fixture
-def files_dir(tmpdir):
-    """Create test dir for testing filenames_from."""
-    with tmpdir.as_cwd():
-        tmpdir.join("a/b/c.py").ensure()
-        tmpdir.join("a/b/d.py").ensure()
-        tmpdir.join("a/b/e/f.py").ensure()
-        yield tmpdir
-
-
-def _normpath(s):
-    return s.replace("/", os.sep)
-
-
-def _normpaths(pths):
-    return {_normpath(pth) for pth in pths}
-
-
-@pytest.mark.usefixtures("files_dir")
-def test_filenames_from_a_directory():
-    """Verify that filenames_from walks a directory."""
-    filenames = set(utils.filenames_from(_normpath("a/b/")))
-    # should include all files
-    expected = _normpaths(("a/b/c.py", "a/b/d.py", "a/b/e/f.py"))
-    assert filenames == expected
-
-
-@pytest.mark.usefixtures("files_dir")
-def test_filenames_from_a_directory_with_a_predicate():
-    """Verify that predicates filter filenames_from."""
-    filenames = set(
-        utils.filenames_from(
-            arg=_normpath("a/b/"),
-            predicate=lambda path: path.endswith(_normpath("b/c.py")),
-        )
-    )
-    # should not include c.py
-    expected = _normpaths(("a/b/d.py", "a/b/e/f.py"))
-    assert filenames == expected
-
-
-@pytest.mark.usefixtures("files_dir")
-def test_filenames_from_a_directory_with_a_predicate_from_the_current_dir():
-    """Verify that predicates filter filenames_from."""
-    filenames = set(
-        utils.filenames_from(
-            arg=_normpath("./a/b"),
-            predicate=lambda path: path == "c.py",
-        )
-    )
-    # none should have matched the predicate so all returned
-    expected = _normpaths(("./a/b/c.py", "./a/b/d.py", "./a/b/e/f.py"))
-    assert filenames == expected
-
-
-@pytest.mark.usefixtures("files_dir")
-def test_filenames_from_a_single_file():
-    """Verify that we simply yield that filename."""
-    filenames = set(utils.filenames_from(_normpath("a/b/c.py")))
-    assert filenames == {_normpath("a/b/c.py")}
-
-
-def test_filenames_from_a_single_file_does_not_exist():
-    """Verify that a passed filename which does not exist is returned back."""
-    filenames = set(utils.filenames_from(_normpath("d/n/e.py")))
-    assert filenames == {_normpath("d/n/e.py")}
-
-
-def test_filenames_from_exclude_doesnt_exclude_directory_names(tmpdir):
-    """Verify that we don't greedily exclude subdirs."""
-    tmpdir.join("1").ensure_dir().join("dont_return_me.py").ensure()
-    tmpdir.join("2").join("1").ensure_dir().join("return_me.py").ensure()
-    exclude = [tmpdir.join("1").strpath]
-
-    # This acts similar to src.flake8.checker.is_path_excluded
-    def predicate(pth):
-        return utils.fnmatch(os.path.abspath(pth), exclude)
-
-    with tmpdir.as_cwd():
-        filenames = list(utils.filenames_from(".", predicate))
-    assert filenames == [os.path.join(".", "2", "1", "return_me.py")]
-
-
-def test_parameters_for_class_plugin():
-    """Verify that we can retrieve the parameters for a class plugin."""
-
-    class FakeCheck:
-        def __init__(self, tree):
-            raise NotImplementedError
-
-    plugin = plugin_manager.Plugin("plugin-name", object())
-    plugin._plugin = FakeCheck
-    assert utils.parameters_for(plugin) == {"tree": True}
-
-
-def test_parameters_for_function_plugin():
-    """Verify that we retrieve the parameters for a function plugin."""
-
-    def fake_plugin(physical_line, self, tree, optional=None):
-        raise NotImplementedError
-
-    plugin = plugin_manager.Plugin("plugin-name", object())
-    plugin._plugin = fake_plugin
-    assert utils.parameters_for(plugin) == {
-        "physical_line": True,
-        "self": True,
-        "tree": True,
-        "optional": False,
-    }
-
-
 def read_diff_file(filename):
     """Read the diff file in its entirety."""
     with open(filename) as fd:
@@ -332,13 +219,6 @@ def test_parse_unified_diff(diff, parsed_diff):
     assert utils.parse_unified_diff(diff) == parsed_diff
 
 
-def test_matches_filename_for_excluding_dotfiles():
-    """Verify that `.` and `..` are not matched by `.*`."""
-    logger = logging.Logger(__name__)
-    assert not utils.matches_filename(".", (".*",), "", logger)
-    assert not utils.matches_filename("..", (".*",), "", logger)
-
-
 def test_stdin_get_value_crlf():
     """Ensure that stdin is normalized from crlf to lf."""
     stdin = io.TextIOWrapper(io.BytesIO(b"1\r\n2\r\n"), "UTF-8")
@@ -351,3 +231,19 @@ def test_stdin_unknown_coding_token():
     stdin = io.TextIOWrapper(io.BytesIO(b"# coding: unknown\n"), "UTF-8")
     with mock.patch.object(sys, "stdin", stdin):
         assert utils.stdin_get_value.__wrapped__() == "# coding: unknown\n"
+
+
+@pytest.mark.parametrize(
+    ("s", "expected"),
+    (
+        ("", ""),
+        ("my-plugin", "my-plugin"),
+        ("MyPlugin", "myplugin"),
+        ("my_plugin", "my-plugin"),
+        ("my.plugin", "my-plugin"),
+        ("my--plugin", "my-plugin"),
+        ("my__plugin", "my-plugin"),
+    ),
+)
+def test_normalize_pypi_name(s, expected):
+    assert utils.normalize_pypi_name(s) == expected
